// This is an Unreal Script
class X2StrategyElement_Reward_Mod extends X2StrategyElement
	dependson(X2RewardTemplate)
	config(GameData);

	
	var localized array<string>				SoldierBlackMarketText;
	var localized string					DoctorPrefixText;
	var config array<int>					SoldierRewardForceLevelGates; 
	var config array<int>					CrewRewardForceLevelGates; // Once force level is passed, rank up crew reward

static function array<X2DataTemplate> CreateTemplates()
{
	local array<X2DataTemplate> Templates;
	Templates.AddItem(CreateFreeCapturedSoldierRewardTemplate());


	return Templates;
}



static function X2DataTemplate CreateFreeCapturedSoldierRewardTemplate()
{
	local X2RewardTemplate Template;

	// Council missions can give you your captured soldiers as a reward, so this template will attempt to
	// do that. Otherwise it defaults to generating a soldier reward
	`CREATE_X2Reward_TEMPLATE(Template, 'Reward_FreeCapturedSoldier');
	Template.rewardObjectTemplateName = 'Soldier';

	Template.IsRewardAvailableFn = IsFreeCapturedSoldierRewardAvailable;
	Template.GenerateRewardFn = GenerateFreeCapturedSoldierReward;
	Template.SetRewardFn = SetPersonnelReward;
	Template.GiveRewardFn = GivePersonnelReward;
	Template.GetRewardStringFn = GetPersonnelRewardString;
	Template.GetBlackMarketStringFn = GetSoldierBlackMarketString;
	Template.GetRewardIconFn = GetGenericRewardIcon;

	return Template;
}

function bool IsFreeCapturedSoldierRewardAvailable()
{
	local XComGameStateHistory History;
	local XComGameState_HeadquartersAlien AlienHQ;

	local XComGameState_Unit NewUnitState;
	local int i;

	// we can only rescue a soldier if there are soldiers to rescue
	History = `XCOMHISTORY;

	AlienHQ = XComGameState_HeadquartersAlien(History.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersAlien'));

		for( i =0; i<AlienHQ.CapturedSoldiers.Length; i++)
		{

			NewUnitState = XComGameState_Unit(History.GetGameStateForObjectID(AlienHQ.CapturedSoldiers[i].ObjectID));
			if(NewUnitState.kAppearance.iGender == eGender_Female)
			{
				return true;
			}
		}
	return false;
}
function GenerateFreeCapturedSoldierReward(XComGameState_Reward RewardState, XComGameState NewGameState, optional float RewardScalar = 1.0, optional StateObjectReference RegionRef)
{
	local XComGameStateHistory History;
	local XComGameState_HeadquartersAlien AlienHQ;
	local XComGameState_Unit NewUnitState;
	local int CapturedSoldierIndex;
	local int i;
	local int nums;
	local XComGameState_Unit CheckUnit;
	local XComGameState_Unit_ModData ModData;

	History = `XCOMHISTORY;

	AlienHQ = XComGameState_HeadquartersAlien(History.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersAlien'));

	// first check if the aliens have captured one of our soldiers. If so, then they get to be the reward
	if(AlienHQ.CapturedSoldiers.Length > 0)
	{

		for( i =0; i<AlienHQ.CapturedSoldiers.Length; i++)
		{
			NewUnitState = XComGameState_Unit(History.GetGameStateForObjectID(AlienHQ.CapturedSoldiers[i].ObjectID));
			if(NewUnitState.kAppearance.iGender == eGender_Female)
			{
				CapturedSoldierIndex =i;
				break;
			}
		}


		
		// mark the soldier is uncaptured
		NewUnitState = XComGameState_Unit(NewGameState.CreateStateObject(class'XComGameState_Unit', AlienHQ.CapturedSoldiers[CapturedSoldierIndex].ObjectID));
		NewUnitState.bCaptured = false;

		CheckUnit = XComGameState_Unit(History.GetGameStateForObjectID(AlienHQ.CapturedSoldiers[CapturedSoldierIndex].ObjectID));
		ModData = class'XComGameState_Unit_ModData'.static.GetModDataComponent(CheckUnit);
		if(ModData == none)
		{
			ModData = XComGameState_Unit_ModData(NewGameState.CreateStateObject(class'XComGameState_Unit_ModData'));
			ModData.Sex =1;
			NewUnitState.AddComponentObject(ModData);
		}
		else
		{
			nums = ModData.Sex +1;
			ModData = XComGameState_Unit_ModData(NewGameState.CreateStateObject(class'XComGameState_Unit_ModData', ModData.ObjectID));
			ModData.Sex = nums;

		}

		NewGameState.AddStateObject(NewUnitState);
		NewGameState.AddStateObject(ModData);

		// remove the soldier from the captured unit list
		AlienHQ = XComGameState_HeadquartersAlien(NewGameState.CreateStateObject(class'XComGameState_HeadquartersAlien', AlienHQ.ObjectID));
		AlienHQ.CapturedSoldiers.Remove(CapturedSoldierIndex, 1);
		NewGameState.AddStateObject(AlienHQ);

		RewardState.RewardObjectReference = NewUnitState.GetReference();
	}
	else
	{
		// somehow the soldier to be rescued has been pulled out from under us! Generate one as a fallback.
		GeneratePersonnelReward(RewardState, NewGameState, RewardScalar, RegionRef);
	}
}
function GeneratePersonnelReward(XComGameState_Reward RewardState, XComGameState NewGameState, optional float RewardScalar = 1.0, optional StateObjectReference RegionRef)
{
	local XComGameState_HeadquartersResistance ResistanceHQ;
	local XComGameStateHistory History;
	local XComGameState_Unit NewUnitState;
	local XComGameState_WorldRegion RegionState;
	local int i, idx, NewRank;
	local name nmCountry;

	History = `XCOMHISTORY;

	// Grab the region and pick a random country
	nmCountry = '';
	RegionState = XComGameState_WorldRegion(History.GetGameStateForObjectID(RegionRef.ObjectID));

	if(RegionState != none)
	{
		nmCountry = RegionState.GetMyTemplate().GetRandomCountryInRegion();
	}

	//Use the character pool's creation method to retrieve a unit
	NewUnitState = `CHARACTERPOOLMGR.CreateCharacter(NewGameState, class'XComGameState_HeadquartersXCom'.default.RewardUnitCharacterPoolSelectionMode, RewardState.GetMyTemplate().rewardObjectTemplateName, nmCountry);
	NewUnitState.RandomizeStats();
	NewUnitState.GiveRandomPersonality();
	NewGameState.AddStateObject(NewUnitState);

	if(RewardState.GetMyTemplate().rewardObjectTemplateName == 'Soldier')
	{
		ResistanceHQ = XComGameState_HeadquartersResistance(History.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersResistance'));

		if(!NewGameState.GetContext().IsStartState())
		{
			ResistanceHQ = XComGameState_HeadquartersResistance(NewGameState.CreateStateObject(class'XComGameState_HeadquartersResistance', ResistanceHQ.ObjectID));
			NewGameState.AddStateObject(ResistanceHQ);
		}
		
		NewUnitState.ApplyInventoryLoadout(NewGameState);
		NewRank = GetPersonnelRewardRank(true, (RewardState.GetMyTemplateName() == 'Reward_Rookie'));
		NewUnitState.SetXPForRank(NewRank);
		NewUnitState.StartingRank = NewRank;
		for(idx = 0; idx < NewRank; idx++)
		{
			// Rank up to squaddie
			if(idx == 0)
			{
				NewUnitState.RankUpSoldier(NewGameState, ResistanceHQ.SelectNextSoldierClass());
				NewUnitState.ApplySquaddieLoadout(NewGameState);
				for(i = 0; i < NewUnitState.GetSoldierClassTemplate().GetAbilityTree(0).Length; ++i)
				{
					NewUnitState.BuySoldierProgressionAbility(NewGameState, 0, i);
				}
			}
			else
			{
				NewUnitState.RankUpSoldier(NewGameState, NewUnitState.GetSoldierClassTemplate().DataName);
			}
		}
	}
	else
	{
		NewUnitState.SetSkillLevel(GetPersonnelRewardRank(false));
	}

	RewardState.RewardObjectReference = NewUnitState.GetReference();
}

function string GetGenericRewardIcon(XComGameState_Reward RewardState)
{
	local XComGameState_Tech TechState;
	local XComGameState_Unit UnitState;
	local StateObjectReference EmptyRef;
	local XComGameStateHistory History;
	local XComGameState_Item ItemState;

	History = `XCOMHISTORY;

	if(RewardState.RewardObjectReference != EmptyRef)
	{
		UnitState = XComGameState_Unit(History.GetGameStateForObjectID(RewardState.RewardObjectReference.ObjectID));

		if(UnitState != none)
		{
			if( UnitState.IsAnEngineer() )  	
				return class'UIUtilities_Image'.const.AlertIcon_Engineering;
			else if( UnitState.IsAScientist() )  	
				return class'UIUtilities_Image'.const.AlertIcon_Science;
			else
				return class'UIUtilities_Image'.const.EventQueue_Staff;
		}

		TechState = XComGameState_Tech(History.GetGameStateForObjectID(RewardState.RewardObjectReference.ObjectID));

		if(TechState == none)
		{
			return class'UIUtilities_Image'.const.AlertIcon_Science;
		}
		
		ItemState = XComGameState_Item(History.GetGameStateForObjectID(RewardState.RewardObjectReference.ObjectID));

		if (ItemState != none)
		{
			return ItemState.GetMyTemplate().strImage;
		}
	}
	return "";
}

function SetPersonnelReward(XComGameState_Reward RewardState, optional StateObjectReference RewardObjectRef, optional int Amount)
{
	RewardState.RewardObjectReference = RewardObjectRef;
}
function GivePersonnelReward(XComGameState NewGameState, XComGameState_Reward RewardState, optional StateObjectReference AuxRef, optional bool bOrder = false, optional int OrderHours = -1)
{
	local XComGameState_HeadquartersXCom XComHQ;
	local XComGameStateHistory History;	
	local XComGameState_Unit UnitState;

	History = `XCOMHISTORY;	

	XComHQ = XComGameState_HeadquartersXCom(History.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersXCom'));
	XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
	NewGameState.AddStateObject(XComHQ);	

	UnitState = XComGameState_Unit(NewGameState.GetGameStateForObjectID(RewardState.RewardObjectReference.ObjectID));
	if(UnitState == none)
	{
		UnitState = XComGameState_Unit(NewGameState.CreateStateObject(class'XComGameState_Unit', RewardState.RewardObjectReference.ObjectID));
		NewGameState.AddStateObject(UnitState);
	}
		
	`assert(UnitState != none);

	if(UnitState.GetMyTemplate().bIsSoldier)
	{
		UnitState.ApplyBestGearLoadout(NewGameState);
	}

	if(bOrder)
	{
		XComHQ.OrderStaff(UnitState.GetReference(), OrderHours);
	}
	else
	{
		XComHQ.AddToCrew(NewGameState, UnitState);
		XComHQ.HandlePowerOrStaffingChange(NewGameState);
	}

	NewGameState.AddStateObject(UnitState);
}
function string GetPersonnelRewardString(XComGameState_Reward RewardState)
{
	local XComGameStateHistory History;
	local XComGameState_Unit UnitState;

	History = `XCOMHISTORY;
	UnitState = XComGameState_Unit(History.GetGameStateForObjectID(RewardState.RewardObjectReference.ObjectID));

	if(UnitState != none)
	{
		if(UnitState.IsASoldier())
		{
			if(UnitState.GetRank() > 0)
			{
				return UnitState.GetName(eNameType_RankFull) @ "-" @ UnitState.GetSoldierClassTemplate().DisplayName;
			}
			else
			{
				return UnitState.GetName(eNameType_RankFull);
			}
		}
		else
		{
			//return default.DoctorPrefixText @ UnitState.GetName(eNameType_Full) @ "-" @ default.SkillLevelText @ string(UnitState.GetSkillLevel()) @ RewardState.GetMyTemplate().DisplayName;
			return default.DoctorPrefixText @ UnitState.GetName(eNameType_Full) @ "-" @ RewardState.GetMyTemplate().DisplayName;
		}
	}

	return "";
}

function string GetSoldierBlackMarketString(XComGameState_Reward RewardState)
{
	return default.SoldierBlackMarketText[`SYNC_RAND_STATIC(default.SoldierBlackMarketText.Length)];
}

function int GetPersonnelRewardRank(bool bIsSoldier, optional bool bIsRookie = false)
{
	local XComGameStateHistory History;
	local XComGameState_HeadquartersAlien AlienHQ;
	local int NewRank, idx;

	History = `XCOMHISTORY;
	AlienHQ = XComGameState_HeadquartersAlien(History.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersAlien'));
	NewRank = 1;

	if(bIsSoldier)
	{
		if (bIsRookie)
		{
			return 0;
		}

		for(idx = 0; idx < default.SoldierRewardForceLevelGates.Length; idx++)
		{
			if(AlienHQ.GetForceLevel() >= default.SoldierRewardForceLevelGates[idx])
			{
				NewRank++;
			}
		}
	}
	else
	{
		for(idx = 0; idx < default.CrewRewardForceLevelGates.Length; idx++)
		{
			if(AlienHQ.GetForceLevel() >= default.CrewRewardForceLevelGates[idx])
			{
				NewRank++;
			}
		}
	}

	return NewRank;
}
